// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI64/16AI64_Linux_3.x.x.x_DN/samples/rxrate/perform.c $
// $Rev: 51835 $
// $Date: 2022-10-17 16:38:20 -0500 (Mon, 17 Oct 2022) $

// 16AI64: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_rx[_1M];



//*****************************************************************************
static int _rate_test(const args_t* args, int seconds, long get)
{
	struct timeval	begin;
	int				empty_count	= 0;
	int				errs		= 0;
	long			got;
	long long		limit		= (long long) args->receive_mb * 1000000L;
	s32				ll;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total		= 0;
	long			us			= 1000000L * seconds;

	gsc_label("Reading");

	errs	+= ai64_query(args->fd, -1, 0, AI64_QUERY_LOW_LATENCY, &ll);
	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + seconds;
	minimum.tv_usec	= begin.tv_usec;

	if (ll)
		printf("SKIPPED  (Low Latency Board)\n");

	for (;ll == 0;)
	{
		got	= ai64_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", errno %d)\n", errno);
			break;
		}

		if (got == 0)
		{
			empty_count++;

			if (empty_count >= 10)
			{
				errs++;
				printf("FAIL <--- (read");
				printf(", too many empty read calls: %d)\n", empty_count);
				ai64_reg_list(args->fd, 1);
				break;
			}
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);
	int		seconds	= 5;

	errs	+= ai64_config_ai	(args->fd, -1, 1, AI64_AI_SCAN_SIZE_SINGLE, 500000L);
	errs	+= ai64_rx_io_mode	(args->fd, -1, 1, args->io_mode, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", seconds, (long) args->receive_mb);
	gsc_label_level_inc();

	errs	+= ai64_ai_buf_clear(args->fd, -1, 1);

	errs	+= _rate_test(args, seconds, get);

	gsc_label_level_dec();
	return(errs);
}



