// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AI64/16AI64_Linux_3.x.x.x_DN/samples/savedata/perform.c $
// $Rev: 51836 $
// $Date: 2022-10-17 16:46:58 -0500 (Mon, 17 Oct 2022) $

// 16AI64: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024L * 1024L)



// variables *******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	s32		active;
	char	buf[64];
	int		errs	= 0;
	int		ret;
	s32		single	= -1;

	gsc_label("Input Channels");

	active	= -1;
	ret		= ai64_ioctl(args->fd, AI64_IOCTL_AI_SCAN_SIZE,	&active	);
	errs	+= ret ? 1 : 0;
	ret		= ai64_ioctl(args->fd, AI64_IOCTL_CHAN_SINGLE,	&single	);
	errs	+= ret ? 1 : 0;

	if (errs == 0)
	{
		switch (active)
		{
			default:

				errs++;
				printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);
				break;

			case AI64_AI_SCAN_SIZE_SINGLE:

				chans[0]	= 1;
				sprintf(buf, "# %ld", (long) single);
				break;

			case AI64_AI_SCAN_SIZE_0_1:

				chans[0]	= 2;
				strcpy(buf, "#s 0-1");
				break;

			case AI64_AI_SCAN_SIZE_0_3:

				chans[0]	= 4;
				strcpy(buf, "#s 0-3");
				break;

			case AI64_AI_SCAN_SIZE_0_7:

				chans[0]	= 8;
				strcpy(buf, "#s 0-7");
				break;

			case AI64_AI_SCAN_SIZE_0_15:

				chans[0]	= 16;
				strcpy(buf, "#s 0-15");
				break;

			case AI64_AI_SCAN_SIZE_0_31:

				chans[0]	= 32;
				strcpy(buf, "#s 0-31");
				break;

			case AI64_AI_SCAN_SIZE_0_63:

				chans[0]	= 64;
				strcpy(buf, "#s 0-63");
				break;
		}

		if (errs == 0)
		{
			printf(	"%ld Channel%s  (%s)\n",
					(long) chans[0],
					(chans[0] == 1) ? "" : "s",
					buf);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args, s32 ll)
{
	int		errs	= 0;
	long	get		= sizeof(_buffer) / 4;
	int		got;

	gsc_label("Reading");

	if (ll)
	{
		printf("SKIPPED  (Low Latency Board)\n");
	}
	else
	{
		got	= ai64_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs	= 1;
		}
		else if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld samples, requested %ld)\n",
					(long) got,
					(long) get);
		}
		else
		{
			errs	= 0;
			printf(	"PASS  (%ld samples)\n",
					(long) get);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(s32 ll, int chan, int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		if (ll)
		{
			printf("SKIPPED  (Low Latency Board)\n");
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			if ((l) && ((l % chan) == 0))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}

			i	= fprintf(file, "  %08lX", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 64;
	int	errs	= 0;
	s32	ll;

	errs	+= ai64_query(args->fd, -1, 1, AI64_QUERY_LOW_LATENCY, &ll);
	errs	+= ai64_config_ai(args->fd, -1, 1, AI64_AI_SCAN_SIZE_0_31, 350000L / 32);
	errs	+= ai64_rx_io_mode(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= _channels(args, &chans);
	errs	+= ai64_ai_buf_clear(args->fd, -1, 1);
	errs	+= _read_data(args, ll);
	errs	+= _save_data(ll, chans, errs);

	return(errs);
}



